"""
File: compare.py
Purpuse: Shape Comparison Algorithm
Tab Width: 4
Revision: $Id: compare.py 531 2006-04-02 00:37:37Z nirh $
"""

from OpenGL import GL

import steams
import Options

class ShapeInfo:
	def __init__(self, image, factor):
		self.pointList = []
		self.outOfBoundsList = []
		
		for x in xrange(0, image.get_width()):
			for y in xrange(0, image.get_height()):
				color = image.get_at((x, y))
				color = color[0:3]
				if color == (0,0,255):
					self.pointList.append((factor*x, factor*y))
				elif color == (255,0,0):
					self.outOfBoundsList.append((factor*x, factor*y))
				elif color == (255,255,255) or color == (0,0,0):
					pass
				else:
					raise Exception("Invalid shape image at %d,%d" % (x,y))
	def getPointList(self):
		return self.pointList
	def getOutOfBoundList(self):
		return self.outOfBoundsList

class CompareConfig:
	def __init__(self, Scale, Position):
		self.Scale = Scale
		self.Position = Position
	def ChangeScale(self, RelativeScale):
		return CompareConfig(self.Scale+RelativeScale/100.0, self.Position) #note the + and not *
	def MoveHorizontal(self, offset):
		return CompareConfig(self.Scale, (self.Position[0]+offset,self.Position[1]) )
	def MoveVertical(self, offset):
		return CompareConfig(self.Scale, (self.Position[0],self.Position[1]+offset) )
	def ApplyTo(self, x,y):
		x *= self.Scale
		y *= self.Scale
		x += self.Position[0]
		y += self.Position[1]
		return (x,y)

class ParticleMap:
	def __init__(self, particles):		
		#prepare matrix for rendering
		## Note that this is different from the other matrix initializations, because we want to make it "upside-down"
		GL.glPushMatrix()
		GL.glLoadIdentity()
		GL.glTranslated(-1,-1,0)
		GL.glScalef(2.0/Options.WindowWidth,2.0/Options.WindowHeight,1)
		
		#render the particles to the back screen buffer using opengl
		GL.glDisable(GL.GL_BLEND)
		GL.glDisable(GL.GL_TEXTURE_2D)
		GL.glClear(GL.GL_COLOR_BUFFER_BIT)
		for p in particles.getList():
			p.draw()
		GL.glPopMatrix()
		
		#copy the image from the back screen buffer to a texture
		GL.glFlush()
		self.pixels = GL.glReadPixels(
			0,0, Options.WindowWidth,Options.WindowHeight,
			GL.GL_LUMINANCE, GL.GL_UNSIGNED_BYTE)
		
		if False:
			minX,minY = 10000,10000
			maxX,maxY = -1,-1
			for pos in xrange(len(self.pixels)):
				if ord(self.pixels[pos]) != 0:
					x,y = pos%Options.WindowWidth, pos/Options.WindowWidth
					if x>maxX:
						maxX = x
					if y>maxY:
						maxY = y
					if x<minX:
						minX = x
					if y<minY:
						minY = y
					
			print "MinX: %d, MaxX: %d, MinY: %d, MaxY: %d" % (minX,maxX,minY,maxY)
	
	def checkPosition(self, compareConfig, x, y):
		x,y = map(int,compareConfig.ApplyTo(x,y))
		if x < 0 or x >= Options.WindowWidth:
			return 0
		if y < 0 or y >= Options.WindowHeight:
			return 0
		pixel = self.pixels[x + y*Options.WindowWidth]
		return ord(pixel)

def _Compare(shapeInfo, compareConfig, map):
	score = 0
	for (x,y) in shapeInfo.getPointList():
		if map.checkPosition(compareConfig, x, y) > 0:
			score += 1
	if score == 0: #still
		return None
	for (x,y) in shapeInfo.getOutOfBoundList():
		if map.checkPosition(compareConfig, x, y) > 0:
			score -= 1
	return score

def tryCompare(gameLevel,image, factor):
	#Create a particle map
	map = ParticleMap(gameLevel.steam)
	
	#prepare the shape information
	shapeInfo = ShapeInfo(image,factor)
	
	#initializing the initial shape config
	initialConfigPosition = list(gameLevel.steam.getCenterOfMass())
	imageSize = image.get_size()
	initialConfigPosition[0] -= imageSize[0] * factor / 2
	initialConfigPosition[1] -= imageSize[1] * factor / 2
	currentConfig = CompareConfig(1.0, initialConfigPosition)
	currentScore = _Compare(shapeInfo,currentConfig,map)
	if currentScore == None:
		return None
	
	stepSize = 16.0 #power of two, preferrably
	keepTrying = True
	while keepTrying:
		keepTrying = False
		
		configurations = []
		configurations += [currentConfig.MoveHorizontal(stepSize),currentConfig.MoveHorizontal(-stepSize)]
		configurations += [currentConfig.MoveVertical(stepSize),currentConfig.MoveVertical(-stepSize)]
		configurations += [currentConfig.ChangeScale(stepSize)]
		if currentConfig.Scale > 0.75:
			configurations += [currentConfig.ChangeScale(-stepSize)]
		
		for config in configurations:
			score = _Compare(shapeInfo,config,map)
			if score != None and score > currentScore:
				currentConfig = config
				currentScore = score
				keepTrying = True
		
		if not keepTrying and stepSize>1:
			keepTrying = True
			stepSize /= 2.0
	
	if currentScore > 0:
		return currentScore,currentConfig
	else:
		return None
