"""
File: level.py
Purpuse: Level Screen implementation and LevelConfig class
Tab Width: 4
Revision: $Id: level.py 544 2006-04-02 01:35:56Z nirh $
"""

### Imports ###
import game
import Options
import os
import pygame
import sprite
import train
import compare
import random

### class Track ###
class Track(object):
	def __init__(self, level, trackPathFile):
		self.level = level
		self.trackPathFile = trackPathFile
		
		self.createTrain()
		self.level.sprites += self.train
	
	def createTrain(self):
		speed = 60
		if random.random()>0.5:
			speed *= -1
		self.train = train.Train(
			self.level,
			self.level.getThemeFile('locomotive.png'), 
			self.level.getThemeFile(self.trackPathFile),speed)
	
	def doFrame(self, timeDelta):
		if self.train.isVisible() == False:
			self.level.sprites -= self.train
			self.createTrain()
			self.level.sprites += self.train
	
	def __del__(self):
		self.level.sprites -= self.train

### class GameLevel ###
class GameLevel(game.GameScreen):
	def __init__(self, levelNumber):
		super(GameLevel, self).__init__()
		
		# Load configuration
		self.config.loadFile(Options.LevelConfigFile % (levelNumber, ))
		self.themePath = Options.ThemePathFormat % (self.config.Theme, )
		
		#Background
		self.sprites += sprite.Sprite(self.getThemeFile('gamebg.png'), visible = True)
		
		shapeImageFile = Options.shapeImage % self.config.Shape
		
		# Create the shape panel
		shapePanel = sprite.Sprite(shapeImageFile, visible = True, blendingEnabled=True)
		shapePanel.setPosition((Options.WindowWidth - shapePanel.getWidth(), 0))
		self.sprites += shapePanel
		
		#shapeGhost
		self.shapeGhost = sprite.Sprite(shapeImageFile,blendingEnabled=True)
		self.sprites += self.shapeGhost
		
		#train tracks
		for line in file(self.getThemeFile('tracks.txt'),"r").readlines():
			line = line.strip()
			if line != None:
				self.tracks = [Track(self,line)]
		
		#self.steam.createRandomSteam()
		
		self.shapeInfoImage = pygame.image.load(Options.shapeInfo % self.config.Shape)
	
	def doFrame(self,timeDelta):
		super(GameLevel, self).doFrame(timeDelta)
		for t in self.tracks:
			t.doFrame(timeDelta)
	
	def getThemeFile(self, fileName):
		return os.path.join(self.themePath, fileName)
	
	def compareImages(self):
		CompareResult = compare.tryCompare(self, self.shapeInfoImage, 4.0)
		if None != CompareResult:
			score, shapePosition = CompareResult
			print "Compare score: ",score,"\t\t"
			self.shapeGhost.setPosition(shapePosition.Position)
			self.shapeGhost.setScale((shapePosition.Scale*4,shapePosition.Scale*4))
			self.shapeGhost.setTimeOut(1.0)
			self.shapeGhost.setVisible(True)
			if score > self.highScore:
				self.highScore = score
		else:
			print "No match was found"
			self.shapeGhost.setVisible(False)
	
	def onKeyDown(self, event, keyAscii):
		if keyAscii.lower() == 'c':
			self.compareImages()
			return
		if keyAscii.lower() == 'q' or keyAscii.lower() == ' ':
			self.screenActive = False
		super(GameLevel, self).onKeyDown(event, keyAscii)
	
	def onMouseClick(self, newX, newY, button, state):
		super(GameLevel, self).onMouseClick(newX, newY, button, state)
		if button == 3:
			self.compareImages()
